@@ -1,11 +1,18 @@
 #!/usr/bin/python3
 
+from datetime import datetime
 import os
 import re
 from debian.changelog import Changelog
-from debmutate.control import ControlEditor, get_relation
+from debmutate.control import (
+    ControlEditor,
+    get_relation,
+    parse_standards_version,
+    )
 from debian.copyright import Copyright, NotMachineReadableError
 from debian.deb822 import Deb822
+from lintian_brush.fixer import report_result, fixed_lintian_tag
+from lintian_brush.standards_version import iter_standards_versions
 
 # Dictionary mapping source and target versions
 upgrade_path = {
@@ -142,6 +149,38 @@ with ControlEditor() as updater:
         # Huh, no standards version?
         pass
     else:
+        try:
+            svs = dict(iter_standards_versions())
+        except FileNotFoundError:
+            dt = None
+            last = None
+            tag = 'out-of-date-standards-version'
+        else:
+            last, last_ts = max(svs.items())
+            last_dt = datetime.fromtimestamp(last_ts)
+            try:
+                ts = svs[parse_standards_version(current_version)]
+            except KeyError:
+                dt = None
+                tag = 'out-of-date-standards-version'
+            else:
+                dt = datetime.fromtimestamp(ts)
+                age = last_dt - dt
+                if age.days > 365 * 2:
+                    tag = 'ancient-standards-version'
+                else:
+                    tag = 'out-of-date-standards-version'
+        fixed_lintian_tag(
+            updater.source,
+            tag,
+            '%s%s%s' % (
+                current_version,
+                (' (released %s)' % dt.strftime('%Y-%m-%d')) if dt else '',
+                (' (current is %s)' %
+                 '.'.join([str(x) for x in last]))
+                if last is not None else '',
+                ))
+
         while current_version in upgrade_path:
             target_version = upgrade_path[current_version]
             try:
@@ -156,7 +195,6 @@ with ControlEditor() as updater:
 
 
 if current_version:
-    print('Update standards version to %s, no changes needed.' %
-          current_version)
-print('Certainty: certain')
-print('Fixed-Lintian-Tags: out-of-date-standards-version')
+    report_result(
+        'Update standards version to %s, no changes needed.' % current_version,
+        certainty='certain')
