@@ -33,6 +33,7 @@ Supported fields:
 - Bug-Submit
 - Screenshots
 - Archive
+- Security-Contact
 
 Extensions for lintian-brush.
 - X-SourceForge-Project
@@ -47,7 +48,6 @@ Supported, but unused.
 - Donation
 - Documentation
 - Registration
-- Security-Contact
 - Webservice
 """
 
@@ -82,6 +82,7 @@ from lintian_brush.vcs import (
     plausible_browse_url as plausible_vcs_browse_url,
     sanitize_url as sanitize_vcs_url,
     is_gitlab_site,
+    determine_browser_url,
     )
 from urllib.request import urlopen, Request
 
@@ -168,6 +169,27 @@ class UpstreamOutput(object):
             self.name == other.name and self.origin == other.origin)
 
 
+class BuildSystem(object):
+    """A build system for an upstream."""
+
+    def __init__(self, name, origin=None):
+        self.name = name
+        self.origin = origin
+
+    def __str__(self):
+        return self.name
+
+    def __repr__(self):
+        return "%s(%r, origin=%r)" % (
+            type(self).__name__, self.name, self.origin)
+
+    def __eq__(self, other):
+        return (
+            isinstance(other, type(self)) and
+            other.name == self.name and
+            other.origin == self.origin)
+
+
 def _load_json_url(http_url: str, timeout: int = DEFAULT_URLLIB_TIMEOUT):
     headers = {'User-Agent': USER_AGENT, 'Accept': 'application/json'}
     http_contents = urlopen(
@@ -447,6 +469,7 @@ def guess_from_pkg_info(path, trust_package):
 
 
 def guess_from_setup_py(path, trust_package):
+    yield BuildSystem('setup.py')
     if not trust_package:
         return
     from distutils.core import run_setup
@@ -897,6 +920,7 @@ def guess_from_doap(path, trust_package):
 
 
 def guess_from_cabal(path, trust_package=False):
+    yield BuildSystem('cabal')
     # TODO(jelmer): Perhaps use a standard cabal parser in Python?
     # The current parser is not really correct, but good enough for our needs.
     # https://www.haskell.org/cabal/release/cabal-1.10.1.0/doc/users-guide/
@@ -1061,6 +1085,7 @@ def guess_from_cargo(path, trust_package):
     except TomlDecodeError as e:
         warn('Error parsing toml file %s: %s' % (path, e))
         return
+    yield BuildSystem('cargo')
     try:
         package = cargo['package']
     except KeyError:
@@ -1084,6 +1109,7 @@ def guess_from_cargo(path, trust_package):
 
 
 def guess_from_pom_xml(path, trust_package=False):
+    yield BuildSystem('maven')
     # Documentation: https://maven.apache.org/pom.html
 
     import xml.etree.ElementTree as ET
@@ -1172,6 +1198,15 @@ def guess_from_get_orig_source(path, trust_package=False):
                     yield UpstreamDatum('Repository', url, 'likely')
 
 
+# https://docs.github.com/en/free-pro-team@latest/github/\
+# managing-security-vulnerabilities/adding-a-security-policy-to-your-repository
+def guess_from_security_md(path, trust_package=False):
+    if path.startswith('./'):
+        path = path[2:]
+    # TODO(jelmer): scan SECURITY.md for email addresses/URLs with instructions
+    yield UpstreamDatum('X-Security-MD', path, 'certain')
+
+
 def _get_guessers(path, trust_package=False):
     CANDIDATES = [
         ('debian/watch', guess_from_debian_watch),
@@ -1190,6 +1225,9 @@ def _get_guessers(path, trust_package=False):
         ('pom.xml', guess_from_pom_xml),
         ('.git/config', guess_from_git_config),
         ('debian/get-orig-source.sh', guess_from_get_orig_source),
+        ('SECURITY.md', guess_from_security_md),
+        ('.github/SECURITY.md', guess_from_security_md),
+        ('docs/SECURITY.md', guess_from_security_md),
         ]
 
     # Search for something Python-y
@@ -1283,6 +1321,56 @@ def guess_upstream_info(
             yield entry
 
 
+def get_upstream_info(path, trust_package=False, net_access=False,
+                      consult_external_directory=False, check=False):
+    metadata_items = []
+    requirements = []
+    buildsystem = None
+    for entry in guess_upstream_info(path, trust_package=trust_package):
+        if isinstance(entry, UpstreamDatum):
+            metadata_items.append(entry)
+        elif isinstance(entry, BuildSystem):
+            buildsystem = entry
+        elif isinstance(entry, UpstreamRequirement):
+            requirements.append(entry)
+    metadata = summarize_upstream_metadata(
+        metadata_items, '.', net_access=net_access,
+        consult_external_directory=consult_external_directory,
+        check=check)
+    return buildsystem, requirements, metadata
+
+
+def summarize_upstream_metadata(
+        metadata_items, path, net_access=False,
+        consult_external_directory=False, check=False):
+    """Summarize the upstream metadata into a dictionary.
+
+    Args:
+      metadata_items: Iterator over metadata items
+      path: Path to the package
+      trust_package: Whether to trust the package contents and i.e. run
+          executables in it
+      net_access: Whether to allow net access
+      consult_external_directory: Whether to pull in data
+        from external (user-maintained) directories.
+    """
+    upstream_metadata = {}
+    update_from_guesses(
+        upstream_metadata,
+        filter_bad_guesses(metadata_items))
+
+    extend_upstream_metadata(
+        upstream_metadata, path, net_access=net_access,
+        consult_external_directory=consult_external_directory)
+
+    if check:
+        check_upstream_metadata(upstream_metadata)
+
+    fix_upstream_metadata(upstream_metadata)
+
+    return {k: v.value for (k, v) in upstream_metadata.items()}
+
+
 def guess_upstream_metadata(
         path, trust_package=False, net_access=False,
         consult_external_directory=False, check=False):
@@ -1296,22 +1384,11 @@ def guess_upstream_metadata(
       consult_external_directory: Whether to pull in data
         from external (user-maintained) directories.
     """
-    upstream_metadata = {}
-    update_from_guesses(
-        upstream_metadata,
-        filter_bad_guesses(
-            guess_upstream_metadata_items(path, trust_package=trust_package)))
-
-    extend_upstream_metadata(
-        upstream_metadata, path, net_access=net_access,
-        consult_external_directory=consult_external_directory)
-
-    if check:
-        check_upstream_metadata(upstream_metadata)
-
-    fix_upstream_metadata(upstream_metadata)
-
-    return {k: v.value for (k, v) in upstream_metadata.items()}
+    metadata_items = guess_upstream_metadata_items(
+        path, trust_package=trust_package)
+    return summarize_upstream_metadata(
+        metadata_items, path, net_access=net_access,
+        consult_external_directory=consult_external_directory, check=check)
 
 
 def _possible_fields_missing(upstream_metadata, fields, field_certainty):
@@ -1813,20 +1890,38 @@ def _copy_bug_db_field(upstream_metadata, net_access):
     return ret
 
 
+def _extrapolate_security_contact_from_security_md(
+        upstream_metadata, net_access):
+    repository_url = upstream_metadata['Repository']
+    security_md_path = upstream_metadata['X-Security-MD']
+    vcs_url = unsplit_vcs_url(
+        repository_url.value, None, security_md_path.value)
+    security_url = determine_browser_url(None, vcs_url)
+    if security_url is None:
+        return None
+    return UpstreamDatum(
+        'Security-Contact', security_url,
+        certainty=min_certainty(
+            [repository_url.certainty, security_md_path.certainty]),
+        origin=security_md_path.origin)
+
+
 EXTRAPOLATE_FNS = [
-    ('Homepage', 'Repository', _extrapolate_repository_from_homepage),
-    ('Bugs-Database', 'Bug-Database', _copy_bug_db_field),
-    ('Bug-Database', 'Repository', _extrapolate_repository_from_bug_db),
-    ('Repository', 'Repository-Browse',
+    (['Homepage'], 'Repository', _extrapolate_repository_from_homepage),
+    (['Bugs-Database'], 'Bug-Database', _copy_bug_db_field),
+    (['Bug-Database'], 'Repository', _extrapolate_repository_from_bug_db),
+    (['Repository'], 'Repository-Browse',
      _extrapolate_repository_browse_from_repository),
-    ('Repository-Browse', 'Repository',
+    (['Repository-Browse'], 'Repository',
      _extrapolate_repository_from_repository_browse),
-    ('Repository', 'Bug-Database',
+    (['Repository'], 'Bug-Database',
      _extrapolate_bug_database_from_repository),
-    ('Bug-Database', 'Bug-Submit', _extrapolate_bug_submit_from_bug_db),
-    ('Bug-Submit', 'Bug-Database', _extrapolate_bug_db_from_bug_submit),
-    ('X-Download', 'Repository', _extrapolate_repository_from_download),
-    ('Repository', 'Name', _extrapolate_name_from_repository),
+    (['Bug-Database'], 'Bug-Submit', _extrapolate_bug_submit_from_bug_db),
+    (['Bug-Submit'], 'Bug-Database', _extrapolate_bug_db_from_bug_submit),
+    (['X-Download'], 'Repository', _extrapolate_repository_from_download),
+    (['Repository'], 'Name', _extrapolate_name_from_repository),
+    (['Repository', 'X-Security-MD'],
+     'Security-Contact', _extrapolate_security_contact_from_security_md),
 ]
 
 
@@ -1876,14 +1971,22 @@ def extend_upstream_metadata(upstream_metadata, path, minimum_certainty=None,
     changed = True
     while changed:
         changed = False
-        for from_field, to_field, fn in EXTRAPOLATE_FNS:
-            try:
-                from_value = upstream_metadata[from_field]
-            except KeyError:
+        for from_fields, to_field, fn in EXTRAPOLATE_FNS:
+            from_certainties = []
+            for from_field in from_fields:
+                try:
+                    from_value = upstream_metadata[from_field]
+                except KeyError:
+                    from_certainties = None
+                    break
+                from_certainties.append(from_value.certainty)
+            if not from_certainties:
+                # Nope
                 continue
+            from_certainty = min_certainty(from_certainties)
             old_to_value = upstream_metadata.get(to_field)
             if old_to_value is not None and (
-                    certainty_to_confidence(from_value.certainty) >=
+                    certainty_to_confidence(from_certainty) >=
                     certainty_to_confidence(old_to_value.certainty)):
                 continue
             result = fn(upstream_metadata, net_access)
